using EorzeaVotes.Config;
using EorzeaVotes.Utilities;
using Lumina.Excel.GeneratedSheets;

namespace EorzeaVotes.Model;

internal enum Breakdown {
    Country,
    Gender,
    Age,
    ZodiacSign,
    CharacterRace,
    CharacterClan,
    CharacterGender,
    CharacterHomeWorld,
    YearStartedPlaying,
    MyersBriggs,
    MainJob,
}

internal static class BreakdownExt {
    internal static Dictionary<Breakdown, Dictionary<string, string>> KeyMap { get; } = new() {
        [Breakdown.Country] = ISO3166.Country.List.ToDictionary(
            country => country.TwoLetterCode,
            country => country.Name
        ),
        [Breakdown.CharacterRace] = Service.DataManager.GetExcelSheet<Race>()!
            .Where(race => race.RowId != 0)
            .ToDictionary(
                race => race.RowId.ToString(),
                race => race.Feminine.RawString
            ),
        [Breakdown.CharacterClan] = Service.DataManager.GetExcelSheet<Tribe>()!
            .Where(tribe => tribe.RowId != 0)
            .ToDictionary(
                tribe => tribe.RowId.ToString(),
                tribe => tribe.Feminine.RawString
            ),
        [Breakdown.CharacterHomeWorld] = Service.DataManager.GetExcelSheet<World>()!
            .Where(world => world.RowId != 0 && world.IsPublic)
            .ToDictionary(
                world => world.RowId.ToString(),
                world => world.Name.RawString
            ),
        [Breakdown.MyersBriggs] = Enum.GetNames<Mbti>()
            .ToDictionary(
                name => name.ToLowerInvariant(),
                name => name.ToUpperInvariant()
            ),
        [Breakdown.MainJob] = Service.DataManager.GetExcelSheet<ClassJob>()!
            .Where(cj => cj.RowId != 0 && !string.IsNullOrWhiteSpace(cj.Name.RawString))
            .ToDictionary(
                cj => cj.RowId.ToString(),
                cj => Util.TitleCase(cj.Name.RawString)
            ),
    };

    private static Dictionary<Breakdown, string[]> Keys { get; } = new() {
        [Breakdown.Country] = ISO3166.Country.List
            .Select(c => c.TwoLetterCode)
            .ToArray(),
        [Breakdown.CharacterRace] = Service.DataManager.GetExcelSheet<Race>()!
            .Where(race => race.RowId != 0)
            .Select(race => race.RowId.ToString())
            .ToArray(),
        [Breakdown.CharacterClan] = Service.DataManager.GetExcelSheet<Tribe>()!
            .Where(tribe => tribe.RowId != 0)
            .Select(tribe => tribe.RowId.ToString())
            .ToArray(),
        [Breakdown.CharacterHomeWorld] = Service.DataManager.GetExcelSheet<World>()!
            .Where(world => world.RowId != 0 && world.IsPublic)
            .Select(world => world.RowId.ToString())
            .ToArray(),
        [Breakdown.MainJob] = Service.DataManager.GetExcelSheet<ClassJob>()!
            .Where(cj => cj.RowId != 0)
            .Select(cj => cj.RowId.ToString())
            .ToArray(),
    };

    internal static string ToHuman(this Breakdown breakdown) {
        return breakdown switch {
            Breakdown.Country => "Country",
            Breakdown.Gender => "Gender",
            Breakdown.Age => "Age",
            Breakdown.ZodiacSign => "Zodiac sign",
            Breakdown.CharacterRace => "Character race",
            Breakdown.CharacterClan => "Character clan",
            Breakdown.CharacterGender => "Character gender",
            Breakdown.CharacterHomeWorld => "Character home world",
            Breakdown.YearStartedPlaying => "Year started playing",
            Breakdown.MyersBriggs => "Myers-Briggs Type Indicator",
            Breakdown.MainJob => "Main FFXIV job",
            _ => throw new ArgumentException("unknown breakdown", nameof(breakdown)),
        };
    }

    internal static string ToSnakeCase(this Breakdown stat) {
        return stat switch {
            Breakdown.Country => "country",
            Breakdown.Gender => "gender",
            Breakdown.Age => "age",
            Breakdown.ZodiacSign => "zodiac_sign",
            Breakdown.CharacterRace => "character_race",
            Breakdown.CharacterClan => "character_clan",
            Breakdown.CharacterGender => "character_gender",
            Breakdown.CharacterHomeWorld => "character_home_world",
            Breakdown.YearStartedPlaying => "year_started_playing",
            Breakdown.MyersBriggs => "myers_briggs",
            Breakdown.MainJob => "main_job",
            _ => throw new ArgumentException("unknown statistic", nameof(stat)),
        };
    }

    internal static string[] GetBreakdownKeys(this Breakdown breakdown) {
        return breakdown switch {
            Breakdown.Country => Keys[Breakdown.Country],
            Breakdown.Gender => new[] {
                "female",
                "non_binary",
                "male",
            },
            Breakdown.Age => new[] {
                "0 to 9",
                "10 to 19",
                "20 to 29",
                "30 to 39",
                "40 to 49",
                "50 to 59",
                "60 to 69",
                "70 to 79",
                "80 to 89",
                "90 to 99",
                "100+",
            },
            Breakdown.ZodiacSign => new[] {
                "aries",
                "taurus",
                "gemini",
                "cancer",
                "leo",
                "virgo",
                "libra",
                "scorpio",
                "sagittarius",
                "capricorn",
                "aquarius",
                "pisces",
            },
            Breakdown.CharacterRace => Keys[Breakdown.CharacterRace],
            Breakdown.CharacterClan => Keys[Breakdown.CharacterClan],
            Breakdown.CharacterGender => new[] {
                "female",
                "male",
            },
            Breakdown.CharacterHomeWorld => Keys[Breakdown.CharacterHomeWorld],
            Breakdown.YearStartedPlaying => Enumerable.Range(2010, DateTime.UtcNow.Year - 2010 + 1)
                .Select(year => year.ToString())
                .ToArray(),
            Breakdown.MyersBriggs => Enum.GetNames<Mbti>()
                .Select(name => name.ToLowerInvariant())
                .ToArray(),
            Breakdown.MainJob => Keys[Breakdown.MainJob],
            _ => throw new ArgumentException("unknown breakdown", nameof(breakdown)),
        };
    }
}
