﻿using System.Diagnostics;
using Dalamud.Game.ClientState.Conditions;
using Dalamud.IoC;
using Dalamud.Plugin;
using Dalamud.Plugin.Services;
using EorzeaVotes.Config;
using EorzeaVotes.Ui;
using EorzeaVotes.Utilities;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Serialization;

namespace EorzeaVotes;

public class Plugin : IDalamudPlugin {
    internal const string Name = "Eorzea Votes";

    internal static readonly JsonSerializerSettings SerializerSettings = new() {
        ContractResolver = new DefaultContractResolver {
            NamingStrategy = new SnakeCaseNamingStrategy(),
        },
        Converters = {
            new StringEnumConverter {
                NamingStrategy = new SnakeCaseNamingStrategy(),
            },
        },
    };

    [PluginService]
    internal DalamudPluginInterface Interface { get; init; }

    [PluginService]
    internal IClientState ClientState { get; init; }

    [PluginService]
    internal ICommandManager CommandManager { get; init; }

    [PluginService]
    internal ICondition Condition { get; init; }

    [PluginService]
    internal IDataManager DataManager { get; init; }

    [PluginService]
    internal IDutyState DutyState { get; init; }

    [PluginService]
    internal IFramework Framework { get; init; }

    internal Configuration Config { get; }
    internal QuestionManager Manager { get; }
    internal PluginUi Ui { get; }
    private Commands Commands { get; }

    private bool _checkNow = true;
    private int _checkSeconds;
    private Stopwatch Stopwatch { get; } = Stopwatch.StartNew();

    public Plugin() {
        this.Config = this.Interface.GetPluginConfig() as Configuration ?? new Configuration();

        this.Interface.Create<Service>();

        this.Manager = new QuestionManager(this);
        this.Ui = new PluginUi(this);
        this.Commands = new Commands(this);

        this.Framework.Update += this.RunChecker;
        this.ClientState.Login += this.Login;

        if (this.Interface.Reason == PluginLoadReason.Installer && !this.Config.OnBoarded) {
            this.Ui.Visible = true;
        }
    }

    public void Dispose() {
        this.ClientState.Login -= this.Login;
        this.Framework.Update -= this.RunChecker;

        this.Commands.Dispose();
        this.Ui.Dispose();
        this.Manager.Dispose();
    }

    internal void SaveConfig() {
        this.Interface.SavePluginConfig(this.Config);
    }

    private void RunChecker(IFramework framework) {
        if (!this.Config.OpenWhenNew) {
            return;
        }

        var wait = this._checkSeconds > 0
            ? this._checkSeconds
            : Math.Clamp(this.Config.CheckFrequency, Configuration.MinFreq, Configuration.MaxFreq);
        if (!this._checkNow && this.Stopwatch.Elapsed < TimeSpan.FromSeconds(wait)) {
            return;
        }

        this._checkNow = false;
        this._checkSeconds = 0;
        this.Stopwatch.Restart();

        var safeToOpen = !this.DutyState.IsDutyStarted
                         && this.ClientState is {
                             LocalPlayer: not null,
                             IsGPosing: false,
                             IsLoggedIn: true,
                             IsPvPExcludingDen: false,
                         }
                         && this.Condition[ConditionFlag.NormalConditions]
                         // ReSharper disable once RedundantExplicitParamsArrayCreation
                         && !this.Condition.Any(new[] {
                             ConditionFlag.BoundByDuty,
                             ConditionFlag.BoundByDuty56,
                             ConditionFlag.BoundByDuty95,
                             ConditionFlag.CarryingItem,
                             ConditionFlag.CarryingObject,
                             ConditionFlag.Crafting,
                             ConditionFlag.Crafting40,
                             ConditionFlag.DutyRecorderPlayback,
                             ConditionFlag.InCombat,
                             ConditionFlag.InDeepDungeon,
                             ConditionFlag.LoggingOut,
                             ConditionFlag.Occupied,
                             ConditionFlag.Occupied30,
                             ConditionFlag.Occupied33,
                             ConditionFlag.Occupied38,
                             ConditionFlag.Occupied39,
                             ConditionFlag.OccupiedInCutSceneEvent,
                             ConditionFlag.OccupiedInEvent,
                             ConditionFlag.OccupiedInQuestEvent,
                             ConditionFlag.OccupiedSummoningBell,
                             ConditionFlag.OperatingSiegeMachine,
                             ConditionFlag.ParticipatingInCustomMatch,
                             ConditionFlag.Performing,
                             ConditionFlag.PlayingLordOfVerminion,
                             ConditionFlag.PlayingMiniGame,
                             ConditionFlag.TradeOpen,
                             ConditionFlag.Transformed,
                             ConditionFlag.WatchingCutscene,
                             ConditionFlag.WatchingCutscene78,
                         });

        if (!safeToOpen) {
            // try again real quick
            this._checkSeconds = 5;
            return;
        }

        Task.Run(async () => await this.Manager.Check());
    }

    private void Login() {
        Task.Run(async () => await this.Manager.OpenIfNew());
    }
}
