use anyhow::Context;
use axum::http::request::Parts;
use axum::{extract::FromRequestParts, async_trait};
use uuid::Uuid;

use crate::ArcState;
use crate::app_error::AppError;

pub struct User {
    pub id: Uuid,
    pub country: String,
}

#[async_trait]
impl FromRequestParts<ArcState> for User {
    type Rejection = AppError;

    async fn from_request_parts(parts: &mut Parts, state: &ArcState) -> Result<Self, Self::Rejection> {
        let token = parts.headers
            .get("x-api-key")
            .context("missing x-api-key header")?
            .to_str()
            .context("x-api-key is not a valid utf-8 string")?;
        let hashed = {
            let mut hasher = blake3::Hasher::new();
            hasher.update(token.as_bytes());
            hasher.finalize().as_bytes().to_vec()
        };

        let country = parts.headers
            .get("cf-ipcountry")
            .and_then(|val| val.to_str().ok())
            .context("missing cloudflare country header")?
            .to_string();

        let id = sqlx::query_scalar!(
            // language=postgresql
            "select id from users where api_key_hash = $1",
            hashed,
        )
            .fetch_optional(&state.pool)
            .await
            .context("could not check database")?
            .context("no such api key")?;

        Ok(User { id, country })
    }
}
